from __future__ import absolute_import

from appium import webdriver
import time

from shishito.runtime.environment.shishito import ShishitoEnvironment


class ControlEnvironment(ShishitoEnvironment):
    """ Appium control environment. """

    def call_browser(self, config_section):
        """ Start webdriver for given config section. Prepare capabilities for the webdriver. If saucelabs setting has value,
        webdriver will be connected to saucelabs. Otherwise appium_url setting will be used.

        :param str config_section: section in platform/environment.properties config
        :return: created webdriver
        """

        # get browser capabilities
        capabilities = self.get_capabilities(config_section)
        saucelabs = self.shishito_support.get_opt('saucelabs')
        if saucelabs:
            remote_url = 'http://%s@ondemand.saucelabs.com:80/wd/hub' % saucelabs
        else:
            remote_url = self.shishito_support.get_opt('appium_url')

        # get driver
        return self.start_driver(capabilities, remote_url)

    def get_capabilities(self, config_section):
        """ Return dictionary of capabilities for specific config combination.

        :param str config_section: section in platform/environment.properties config
        :return: dict with capabilities
        """

        get_opt = self.shishito_support.get_opt
        return {
            'platformName': get_opt(config_section, 'platformName'),
            'platformVersion': get_opt(config_section, 'platformVersion'),
            'deviceName': get_opt(config_section, 'deviceName'),
            'app': get_opt('app') or get_opt(config_section, 'app'),
            'appiumVersion': get_opt(config_section, 'appiumVersion') or '1.6.5',
            'autoAcceptAlerts': True if get_opt(config_section, 'autoAcceptAlerts').lower() == 'true' else False,
            'name': self.get_test_name() + time.strftime('_%Y-%m-%d')
        }

    def get_pytest_arguments(self, config_section):
        """ Get environment specific arguments for pytest.

        :param config_section: section in platform/environment.properties config
        :return: dict with arguments for pytest or None
        """

        pytest_args = {
            '--platformName': '--platformName=%s' % self.shishito_support.get_opt(config_section, 'platformName'),
            '--platformVersion': '--platformVersion=%s' % self.shishito_support.get_opt(config_section, 'platformVersion'),
            '--deviceName': '--deviceName=%s' % self.shishito_support.get_opt(config_section, 'deviceName'),
            '--autoAcceptAlerts': '--autoAcceptAlerts=%s' % self.shishito_support.get_opt(config_section, 'autoAcceptAlerts'),
            '--app': '--app=%s' % (self.shishito_support.get_opt('app') or self.shishito_support.get_opt(config_section, 'app'))
        }

        saucelabs = self.shishito_support.get_opt('saucelabs')
        if saucelabs:
            pytest_args['--saucelabs'] = '--saucelabs=%s' % saucelabs
        return pytest_args

    def start_driver(self, capabilities, remote_driver_url):
        """ Prepare selenium webdriver.

        :param capabilities: capabilities used for webdriver initialization
        :param remote_driver_url: url to which the driver will be connected
        """

        driver = webdriver.Remote(
            command_executor=remote_driver_url,
            desired_capabilities=capabilities,
        )

        return driver



import os
from importlib import import_module

import pytest
import sys
if sys.version_info.major > 2:
    import configparser
else:
    import ConfigParser
    configparser = ConfigParser


class ShishitoSupport(object):
    """ Support class for getting config values and importing modules according to
    test platform and environment.

    :param cmd_args: dictionary with command lines arguments
    :type cmd_args: dict or None
    :param project_root: test project root (where to find config files, tests, ..)
    :type project_root: str or None
    """

    def __init__(self, cmd_args=None, project_root=None):
        # parsed arguments from command_line
        self.args_config = cmd_args or {}

        # called without cmd_args
        # if True gid will use only pytest.config
        # if False gid uses pytest.config, cmd_args and config files
        self.used_in_test = cmd_args is None

        self.project_root = project_root or self.find_project_root()

        # get configs
        self.configs = self.load_configs()

        self.test_environment = self.get_opt('test_environment')
        self.test_platform = self.get_opt('test_platform')

        # get environment config
        self.env_config = self.get_environment_config()

    def find_project_root(self):
        """ Try to find config directory on sys.path

        :raises ValueError: if config directory is not found on sys.path
        """

        for path in sys.path:
            config_dir = os.path.join(path, 'config')
            tests_dir = os.path.join(path, 'tests')
            if os.path.exists(config_dir) and os.path.exists(tests_dir):
                return path

        raise ValueError('Can not find config dir on sys.path')

    def load_configs(self):
        """ Load variables from .properties configuration files. If local_execution is true in local_config file, function
        return both local config dict and server confgi dict. Otherwise function returns only server config dict.

        :return: list with config dictionaries
        :raises ValueError: if config path does not exist
        """

        config_path = os.path.join(self.project_root, 'config')
        if not os.path.exists(config_path):
            raise ValueError('Configuration path does not exist (%s)' % config_path)

        configs = []

        # load server config variables
        config = configparser.ConfigParser()
        server_config = os.path.join(config_path, 'server_config.properties')
        config.read(server_config)
        server_config_vars = dict(config.defaults())
        configs.append((server_config_vars, 'server config'))

        # load local config variables
        config = configparser.ConfigParser()
        local_config = os.path.join(config_path, 'local_config.properties')
        config.read(local_config)
        local_config_vars = dict(config.defaults())

        if local_config_vars.get('local_execution').lower() == 'true':
            configs.insert(0, (local_config_vars, 'local config'))

        return configs

    def get_opt(self, *args, default=None):
        """ Get value from config variables based on provided key and optionaly section.

        If key is given, function searches in pytest.config, command lines arguments, local config (if enabled) and server config.
        If section and key are given, function searches in environment config file.


        :param args: key or config_section, key
        :param default: default value to return if not found
        :return: value for given key or None
        :raises TypeError: if called with wrong number of arguments
        """

        if len(args) > 2 or not args:
            raise TypeError('Wrong number of arguments (%s), takes 1 or 2.' % len(args))

        if len(args) == 2:
            section, key = args
        else:
            key = args[0]
            section = None

        if section:
            # use env config
            try:
                return self.env_config.get(section, key)
            except configparser.NoOptionError:
                return default

        # first try to lookup pytest config
        try:
            value = pytest.config.getoption(key)
            if value:
                return value
        except (ValueError, AttributeError):
            pass

        # look in cmd args config
        value = self.args_config.get(key)
        if value:
            return value

        for cfg, cfg_name in self.configs:
            value = cfg.get(key)
            if value:
                return value
            else:
                return default

    def get_environment_config(self):
        """ Load environment specific configuration file according to current test platform and environment

        :return: environment configuration file
        :raises ValueError: if config path does not exist
        """

        config = configparser.ConfigParser()
        config_path = os.path.join(self.project_root, 'config', self.test_platform, self.test_environment + '.properties')

        if not os.path.exists(config_path):
            raise ValueError('Config file in location {0} was not found!'.format(config_path))

        config.read(config_path)
        return config

    def get_module(self, module):
        """ Import object from given module according to current test platform and environment.

        :param str module: module used for import
        :return: imported object
        :raises ValueError: in case of unknown module
        """

        platform_execution = 'shishito.runtime.platform.' + self.test_platform + '.control_execution'
        platform_test = 'shishito.runtime.platform.' + self.test_platform + '.control_test'
        test_environment = 'shishito.runtime.environment.' + self.test_environment

        if module == 'platform_execution':
            return getattr(import_module(platform_execution), 'ControlExecution')
        elif module == 'platform_test':
            return getattr(import_module(platform_test), 'ControlTest')
        elif module == 'test_environment':
            return getattr(import_module(test_environment), 'ControlEnvironment')

        raise ValueError('Unknown module.')

    def get_test_control(self):
        """ Return TestControl object for current test platform. Used in tests.

        :return: TestControl object
        """

        return self.get_module('platform_test')()



"""
@author: Vojtech Burian
@summary: Common configuration functions supporting test execution.
Various startup and termination procedures, helper functions etc.
Not to be used for directly testing the system under test (must not contain Asserts etc.)
"""

import time

from shishito.runtime.platform.shishito_control_test import ShishitoControlTest


class ControlTest(ShishitoControlTest):
    """ ControlTest for web platform """

    def stop_browser(self, delete_cookies=True, driver=None):
        """ Webdriver termination function.

        :param bool delete_cookies: delete cookies from webdriver
        """
        if not driver:
            # Close all drivers
            for d in self.drivers:
                if delete_cookies:
                    d.delete_all_cookies()

                d.quit()

            # Cleanup the driver info
            del self.drivers[:]

        else:
            # Close just the specific driver
            if delete_cookies:
                driver.delete_all_cookies()
            driver.quit()

            self.drivers.remove(driver)


    def test_init(self, driver, url=None):
        """ Executed only once after browser starts.
        Suitable for general pre-test logic that do not need to run before every individual test-case.
        Open given url and wait for given time (setting "default_implicit_wait").
        """
        if url:
            driver.get(url)
        driver.implicitly_wait(int(self.shishito_support.get_opt('default_implicit_wait')))

    def start_test(self, reload_page=None):
        """ To be executed before every test-case (test function).

        :param reload_page: it True reloads page and waits
        """

        if reload_page:
            self.drivers[0].get(self.shishito_support.get_opt('base_url'))
            self.drivers[0].implicitly_wait(self.shishito_support.get_opt('default_implicit_wait'))
            time.sleep(5)



"""
@author: Irina Gvozdeva
@summary: Functions that operate over ZAPI API
"""
import json
import requests
from datetime import datetime, timedelta

jira_server = "https://jira.salsitasoft.com"
execution_status = {"PASS": 1, "FAIL": 2, "WIP": 3, "BLOCKED": 4, "UNEXECUTED": -1}


class ZAPI(object):

    def get_projects(self, auth):
        #Get projects from JIRA
        url = "%s/rest/zapi/latest/util/project-list" % jira_server
        r = requests.get(url, auth=auth)
        return r.json["options"]

    def get_project_id(self, project_name, auth):
        #Get project id based on project name
        for project in self.get_projects(auth):
            if project["label"] == project_name:
                return project["value"]

    def get_project_versions(self, project_name, auth):
        #Get project versions, based on project name
        project_id = self.get_project_id(project_name, auth)
        url = "%s/rest/zapi/latest/util/versionBoard-list" % jira_server
        parameters = {"projectId": project_id, "showUnscheduled": "True"}
        r = requests.get(url, auth=auth, params=parameters)
        return r.json

    def get_version_id(self, project_name, version_name, auth):
        #Get version id based on project name and version name
        versions = self.get_project_versions(project_name, auth)
        for version in versions["unreleasedVersions"]:
            if version["label"] == version_name:
                return version["value"]

    # Cycle api
    # Not used !!!
    def get_project_cycles(self, project_name, version, auth):
        #Get project cycle based on project name and version name
        project_id = self.get_project_id(project_name, auth)
        version_id = self.get_version_id(project_name, version, auth)
        url = "%s/rest/zapi/latest/cycle" % jira_server
        parameters = {"projectId": project_id, "versionId": version_id, "offset": 0, "expand": "executionSummaries"}
        r = requests.get(url, auth=auth, params=parameters)
        json_res = r.json
        test_cycle = {}
        for i in json_res.items():
            if i[0].isdigit():
                test_cycle[i[0]] = [i[1]]
        return test_cycle

    def get_cycle_execution_tests(self, cycle_id, auth):
        # Get all executions for cycle with id
        url = "%s/rest/zapi/latest/execution?cycleId=%s" % (jira_server, cycle_id)
        r = requests.get(url, auth=auth)
        json_res = r.json
        return json_res.get('executions', [])

    def get_issueid(self, cycle_id, jira_id, auth):
        # Get issue id based on id from Jira
        issues = self.get_cycle_execution_tests(cycle_id, auth)
        for issue in issues:
            if issue["issueKey"] == jira_id:
                return issue["issueId"]

    def create_new_test_cycle(self, cycle_name, project_name, version_name, auth):
        # Create new test cycle, based on project and version names
        project_id = self.get_project_id(project_name, auth)
        version_id = self.get_version_id(project_name, version_name, auth)
        today = datetime.now()
        end_date = today + timedelta(days=30)
        url = "%s/rest/zapi/latest/cycle" % jira_server
        data = {
            "clonedCycleId": "",
            "name": cycle_name,
            "build": "",
            "environment": "",
            "description": "",
            "startDate": today.strftime("%d/%b/%y"),
            "endDate": end_date.strftime("%d/%b/%y"),
            "projectId": project_id,
            "versionId": version_id
        }
        headers = {'content-type': 'application/json'}
        r = requests.post(url, auth=auth, data=json.dumps(data), headers=headers)
        return r.json["id"] #cycle_id

    def copy_test_cycle(self, cycle_id, cycle_name, project_name, version_name, auth):
        #Copy existing test cycle -didn't use it yet
        project_id = self.get_project_id(project_name, auth)
        version_id = self.get_version_id(project_name, version_name, auth)
        today = datetime.now()
        end_date = today + timedelta(days=30)
        url = "%s/rest/zapi/latest/cycle" % jira_server
        data = {
            "clonedCycleId": cycle_id,
            "name": cycle_name,
            "build": "",
            "environment": "",
            "description": "",
            "startDate": today.strftime("%d/%b/%y"),
            "endDate": end_date.strftime("%d/%b/%y"),
            "projectId": project_id,
            "versionId": version_id
        }
        headers = {'content-type': 'application/json'}
        r = requests.post(url, auth=auth, data=json.dumps(data), headers=headers)
        return r.json["id"] #cycle_id

    def delete_test_cycle(self, cycle_id, auth):
        #Delete existing test cycle use it's id
        url = "%s/rest/zapi/latest/cycle/%s" % (jira_server, cycle_id)
        r = requests.get(url, auth=auth)
        if "Error" not in r.json:
            url = "%s/rest/zapi/latest/cycle/%s" % (jira_server, cycle_id)
            r = requests.delete(url, auth=auth)
            return r.json

    #Execution API
    def add_new_execution(self, project_name, version_name, cycle_id, issue_id, auth):
        # Add new execution to cycle
        project_id = self.get_project_id(project_name, auth)
        version_id = self.get_version_id(project_name, version_name, auth)
        url = "%s/rest/zapi/latest/execution" % jira_server
        data = {
            "issueId": issue_id,
            "versionId": version_id,
            "cycleId": cycle_id,
            "projectId": project_id
        }
        headers = {'content-type': 'application/json'}
        r = requests.post(url, auth=auth, data=json.dumps(data), headers=headers)
        return r.json.keys()[0] #Strange!!!

    def add_tests_to_cycle(self, issue_keys, project_name, version_name, cycle_id, auth):
        project_id = self.get_project_id(project_name, auth)
        version_id = self.get_version_id(project_name, version_name, auth)
        url = "%s/rest/zapi/latest/execution/addTestsToCycle" % jira_server
        data = {
            "issues": issue_keys,
            "versionId": version_id,
            "cycleId": cycle_id,
            "projectId": project_id,
            "method": "1"
        }
        headers = {'content-type': 'application/json'}
        r = requests.post(url, auth=auth, data=json.dumps(data), headers=headers)
        return r.json

    def get_execution_id(self, execution_id, auth):
        url = jira_server + "/rest/zapi/latest/execution/%s" % execution_id
        r = requests.get(url, auth=auth)
        return r.json

    def get_execution_test(self, issue_id, auth):
        # Get all executions for issue_id
        url = "%s/rest/zapi/latest/execution/" % jira_server
        parameters = {"issueId": issue_id}
        r = requests.get(url, auth=auth, params=parameters)
        return r.json

    def update_execution_status(self, execution_id, status, auth):
        #Update execution status
        url = "%s/rest/zapi/latest/execution/%s/quickExecute" % (jira_server, execution_id)
        data = {'status': execution_status[status]}
        headers = {'content-type': 'application/json'}
        r = requests.post(url, auth=auth, data=json.dumps(data), headers=headers)
        return r.status_code



"""
@author: Irina
@summary: Functions that operate over BrowserStack API
"""
import json
import requests
import sys
import time


class BrowserStackAPI(object):

    def request_get_verify(self, auth, url):
        '''
        Returns the response from Browserstack API, if the status code is 200.
        Retries 5 times before failing completely
        :param auth: Browserstack authorization (tuple)
        :param url: API url
        :return: Response from Browserstack (or raise Connection Error)
        '''

        response = None

        for attempt in range(5):

            response = requests.get(url, auth=auth)
            if response.status_code == 200:
                return response

            else:
                print("Browserstack responded with {0} status code".format(response.status_code))
                time.sleep(1)

        #attempts depleted
        raise ConnectionError('Cannot get valid response from Browserstack. Latest status code = {0}'.format(response.status_code))

    def get_projects(self, auth):
        url = "https://www.browserstack.com/automate/projects.json"
        r = self.request_get_verify(auth, url)
        return r.json()


    def get_project(self, auth, project_name):
        for project in self.get_projects(auth):
            if project['automation_project']['name'] == project_name:
                project_id = project['automation_project']['id']
                url = "https://browserstack.com/automate/projects/%s.json" % project_id
                r = self.request_get_verify(auth, url)
                return r.json()

    def get_builds(self, auth):
        url = "https://www.browserstack.com/automate/builds.json"
        r = self.request_get_verify(auth, url)
        return r.json()

    def get_build_hash_id(self, auth, build_name):
        for build in self.get_builds(auth):
            if build['automation_build']['name'] == build_name:
                return build['automation_build']['hashed_id']

    def get_sessions(self, auth, build_name):
        hashed_id = self.get_build_hash_id(auth, build_name)
        url = "https://www.browserstack.com/automate/builds/%s/sessions.json" % hashed_id
        r = self.request_get_verify(auth, url)
        return r.json()

    def get_session(self, auth, build_name, status):
        for session in self.get_sessions(auth, build_name):
            automation_session = session['automation_session']
            if automation_session['status'] == status:
                return automation_session

    def get_session_link(self, session):
        if session:
            session_link = session['logs'][:-4]
            print(session_link) #Printing to stdout, so that it can be added to test report
            return session_link

    def get_session_hashed_id(self, session):
        return session['hashed_id'] if session else None

    def is_session_available(self, auth):
        url = "https://www.browserstack.com/automate/plan.json"
        r = self.request_get_verify(auth, url)
        session = r.json()
        session_running = session["parallel_sessions_running"]
        session_all = session["parallel_sessions_max_allowed"]
        return int(session_all) - int(session_running)

    def change_status(self, auth, session_id):
        url = "https://www.browserstack.com/automate/sessions/%s.json" % session_id
        status = {'status': 'error'}
        headers = {'content-type': 'application/json'}
        requests.put(url, auth=auth, data=json.dumps(status), headers=headers)

    def wait_for_free_sessions(self, auth, wait_total, wait_delay):
        """ Waits for BrowserStack session to be available """
        wait_steps = int(wait_total) / int(wait_delay)
        counter = 0
        session_available = self.is_session_available(auth)
        while not session_available:
            print('No BrowserStack session available. Waiting for %s minutes...' % wait_delay)
            if counter >= wait_steps:
                sys.exit('No BrowserStack session got available'
                         ' after %s minutes. No test will be executed.' % wait_total)
            counter += 1
            time.sleep(int(wait_delay) * 60)
            session_available = self.is_session_available(auth)
        return session_available



from __future__ import absolute_import

from selenium import webdriver
from selenium.webdriver.chrome import options
import os
import sys
from shishito.runtime.environment.shishito import ShishitoEnvironment


class ControlEnvironment(ShishitoEnvironment):
    """ Node-webkit control environment. """

    def call_browser(self, config_section):
        """ Start chromedriver for node-webkit application. Chromedriver have to be located in same
        folder as node-webkit application (AUT).

        :param str config_section: section in platform/environment.properties config
        :return: created webdriver
        """
        node_webkit_chromedriver_path = self.shishito_support.get_opt('node_webkit_chromedriver_path')
        if not node_webkit_chromedriver_path:
            raise ValueError('Path to Chomedriver is not specified, please check configuration file.')

        binary_location = self.shishito_support.get_opt(config_section, 'binary_location')
        if not binary_location:
            raise ValueError('Path "binary_location" of the binary being tested not specified, please check: config/node_webkit/*.properties')

        chrome_options = options.Options()
        chrome_options.binary_location = binary_location

        # get driver
        return self.start_driver(node_webkit_chromedriver_path, chrome_options)



    def start_driver(self, node_webkit_chromedriver_path, chrome_options):
        """ Prepare chromedriver for node-webkit application.

        :param capabilities: capabilities used for webdriver initialization
        """

        driver = webdriver.Chrome(node_webkit_chromedriver_path, chrome_options = chrome_options)

        return driver



# -*- coding: utf-8 -*-

"""Console script for shishito."""
import sys
import click
from shishito.shishito_runner import ShishitoRunner
import os


@click.command()
def main(args=None):
    """Console script for pyforecast."""
    click.echo("This is Shishito test runner")
    click.echo("See click documentation at http://click.pocoo.org/")
    ShishitoRunner(os.getcwd()).run_tests()


if __name__ == "__main__":
    sys.exit(main())


# /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Vojtech Burian
@summary: User authorization selenium library
"""
import time

from selenium.webdriver.common.keys import Keys


def log_in(email_field, email, password_field, password, submit_button=None):
    """ basic login function """
    email_field.clear()
    password_field.clear()

    email_field.send_keys(email)
    password_field.send_keys(password)
    if submit_button:
        submit_button.click()
    else:
        password_field.send_keys(Keys.RETURN)
    time.sleep(1)



"""
@author: Vojtech Burian
@summary: Selenium Webdriver Python test runner
"""
from shishito.runtime.platform.shishito_execution import ShishitoExecution


class ControlExecution(ShishitoExecution):
    """ ControlExecution for web platform """

    def get_test_result_prefix(self, config_section):
        """ Create string prefix for test results.

        :param str config_section: section in platform/environment.properties config
        :return: str with test result prefix
        """

        browser = self.shishito_support.get_opt(config_section, 'browser')
        browser_version = self.shishito_support.get_opt(config_section, 'browser_version')
        resolution = self.shishito_support.get_opt(config_section, 'resolution')

        return '[%s, %s, %s]' % (browser, browser_version, resolution)



import inspect
import os
import re
import configparser
import base64
from selenium import webdriver
import json

OPTIONS = 'OPTIONS'
ARGUMENTS = 'ARGUMENTS'
EXTENSIONS = 'EXTENSIONS'

BROWSER_KEYWORDS = {  # map lowercase browser name to option keyword understood by chromedriver/geckodriver...
    'chrome': {
        OPTIONS: 'chromeOptions',
        ARGUMENTS: 'args',
        EXTENSIONS: 'extensions',
    },
    'firefox': {
        OPTIONS: 'moz:firefoxOptions',
        ARGUMENTS: 'args',
    }
}


class ShishitoEnvironment(object):

    """ Base class for test environment. """


    def __init__(self, shishito_support):
        self.shishito_support = shishito_support


    def get_browser_arguments(self, config_section):
        """
        return array of browser command line arguments from config_section
            [Chrome]
            browser_arguments=--enable-experimental-web-platform-features --unlimited-storage
            ...
        :param config_section: name of config section in web/*.properties (e.g. Chrome)
        :return: array
        """

        if config_section is None:
            return []

        try:
            arguments_string = self.shishito_support.get_opt(config_section, 'browser_arguments')
        except configparser.NoOptionError:
            return []

        if arguments_string:
            arguments = [i for i in re.split('\s+', arguments_string) if i != '']
            return arguments
        else:
            return []

    def get_experimental_arguments(self, config_section):
        """
        return array of browser command line arguments from config_section
            [Chrome]
            experimental_arguments=--mobileEmulation--{"deviceName": "Apple iPhone 6"}
            ...
        :param config_section: name of config section in web/*.properties (e.g. Chrome)
        :return: array
        """

        if config_section is None:
            return []

        try:
            arguments_string = self.shishito_support.get_opt(config_section, 'experimental_arguments')
        except configparser.NoOptionError:
            return []

        if arguments_string:
            arguments = arguments_string.split('--')
            return arguments[1:]
        else:
            return None

    def get_browser_extensions(self, config_section):
        """
        Check browser config for 'extensions' parameter, if not found fallback to common config.
        Return array of browser extension files from config_section.
        If the extension name is a variable ( $VAR_NAME ), get the value from the environment
            [Chrome]
            extensions=/tmp/extensions/my-extension.crx  another_extension.crx
            ...
        :param config_section: name of config section in web/*.properties (e.g. Chrome)
        :return: array
        """

        extension_string = None
        extensions = []
        if config_section is not None:
            try:
                extension_string = self.shishito_support.get_opt(config_section, 'browser_extensions')  # browser config
            except configparser.NoOptionError:
                extension_string = None

        if extension_string is None:
            try:
                extension_string = self.shishito_support.get_opt('browser_extensions')                 # common config
            except configparser.NoOptionError:
                pass

        if extension_string is None:
            return []

        for item in re.split('\s+', extension_string):
            if item != '':
                m = re.match('^\$([A-Z][A-Z_]+)$', item)
                if m is not None:
                    var_name = m.group(1)
                    if var_name not in os.environ:
                        raise Exception("Error getting browser_extensions: env variable '" + item + "' not defined")
                    extensions.append(os.environ[var_name])  # take the extension path as configured
                else:
                    extensions.append(item)     # take the extension path as configured

        return extensions


    def add_cmdline_arguments_to_browser(self, browser_capabilities, config_section):
        """
        Add browser command line arguments to capabilities dict
        :param browser_capabilities: dict (see https://w3c.github.io/webdriver/webdriver-spec.html)
        :param config_section: name of the browser config section (e.g. 'Chrome')
        """

        browser_name = self.shishito_support.get_opt(config_section, 'browser').lower()
        arguments = self.get_browser_arguments(config_section)
        if arguments:
            try:
                options_kw = BROWSER_KEYWORDS[browser_name][OPTIONS]
                args_kw = BROWSER_KEYWORDS[browser_name][ARGUMENTS]
                browser_capabilities.setdefault(options_kw, {}).setdefault(args_kw, []).extend(arguments)
            except:
                pass


    def add_extensions_to_browser(self, browser_capabilities, config_section):
        """
        Add base64-encoded extensions to capabilities dict
        :param browser_capabilities: dict (see https://w3c.github.io/webdriver/webdriver-spec.html)
        :param config_section: name of the browser config section (e.g. 'Chrome')
        """

        browser_name = self.shishito_support.get_opt(config_section, 'browser').lower()
        extensions = self.get_browser_extensions(config_section)
        if extensions:
            try:
                options_kw = BROWSER_KEYWORDS[browser_name][OPTIONS]
                exts_kw = BROWSER_KEYWORDS[browser_name][EXTENSIONS]
                browser_capabilities.setdefault(options_kw, {}).setdefault(exts_kw, [])
            except:
                return

            for extension in extensions:
                with open(extension, 'rb') as ext_file:
                    extension_base64 = base64.b64encode(ext_file.read()).decode('UTF-8')
                browser_capabilities[options_kw][exts_kw].append(extension_base64)

    def add_experimental_option(self, browser_capabilities, config_section):
        browser_name = self.shishito_support.get_opt(config_section, 'browser').lower()
        arguments = self.get_experimental_arguments(config_section)
        if arguments and browser_name == 'chrome':
            chrome_options = webdriver.ChromeOptions()
            if('mobileEmulation' in arguments):
                index = arguments.index('mobileEmulation')
                chrome_options.add_experimental_option(arguments[index], json.loads(arguments[index+1]))
            return browser_capabilities.update(chrome_options.to_capabilities())
        return browser_capabilities

    def call_browser(self, config_section):
        """ Start webdriver for given config section."""
        raise NotImplementedError()

    def get_capabilities(self, config_section):
        """ Return dictionary of capabilities for specific config combination/

        :param str config_section: section in platform/environment.properties config
        :return: dict with capabilities
        Example of browser capabilities object:
             "desiredCapabilities" : {
                "browserName" : "chrome",
                "version" : "50.1",
                "acceptSslCerts" : true,
                "javascriptEnabled" : true,
                "platform" : "ANY",
                "chromeOptions" : {
                   "args" : [ "--ignore-certificate-errors" ],
                   "extensions" : [ "base64-xxxxx" ]
                }
             }
        """
        get_opt = self.shishito_support.get_opt
        test_platform = self.shishito_support.test_platform
        if(test_platform == 'web'):

            # Get logging levels from config
            logging_driver = get_opt(config_section, 'logging_driver', default='WARNING').upper()
            logging_browser = get_opt(config_section, 'logging_browser', default='WARNING').upper()
            logging_performance = get_opt(config_section, 'logging_performance', default='WARNING').upper()

            capabilities = {
                'browserName': get_opt(config_section, 'browser').lower(),
                'version': get_opt(config_section, 'browser_version'),
                'resolution': get_opt(config_section, 'resolution'),
                'javascriptEnabled': True,
                'acceptSslCerts': get_opt('accept_ssl_cert').lower() == 'true',
                'loggingPrefs': {'driver': logging_driver,
                                 'browser': logging_browser,
                                 'performance': logging_performance}
            }
        if(test_platform == 'mobile'):
            capabilities = {
                'browserName': get_opt(config_section, 'browser').lower(),
                'javascriptEnabled': True,
                'acceptSslCerts': get_opt('accept_ssl_cert').lower() == 'true',
            }

        self.add_cmdline_arguments_to_browser(capabilities, config_section)
        self.add_extensions_to_browser(capabilities, config_section)
        self.add_experimental_option(capabilities, config_section)
        return capabilities


    def start_driver(self, browser_type, capabilities, config_section=None):
        """ Prepare selenium webdriver.

        :param str browser_type: type of browser for which prepare driver
        :param dict capabilities: capabilities used for webdriver initialization
        """

        raise NotImplementedError()


    def get_browser_profile(self, browser_type, capabilities, config_section=None):
        """ Return browser profile ready to be passed to driver.

        :param str browser_type: browser type (chrome, firefox, ..)
        :param dict capabilities: capabilities dict - can be updated
        :return: browser profile or None
        """

        # lowercase browser_type
        browser_type = browser_type.lower()
        profile = None

        if browser_type == 'firefox':
            profile = webdriver.FirefoxProfile()
            for ext in self.get_browser_extensions(config_section):
                profile.add_extension(ext)

        return profile


    def get_pytest_arguments(self, config_section):
        """ Get environment specific arguments for pytest.

        :param config_section: section in platform/environment.properties config
        :return: dict with arguments for pytest or None
        """
        pass

    def get_test_name(self):
        """ Return test name from the call stack, assuming there can be only
        one "test\_" file in the stack. If there are more it means two PyTest
        tests ran when calling get_test_name, which is invalid use case.

        :return: test name or project_name setting
        """

        frames = inspect.getouterframes(inspect.currentframe())
        for frame in frames:
            if re.match('test_.*', os.path.basename(frame[1])):
                return os.path.basename(frame[1])[:-3]

        return self.shishito_support.get_opt('project_name')



""" report test results in HTML format
author: Irina Gvozdeva
Based on initial code from Ross Lawley and mozilla selenium report generating.
"""
import glob

import pkg_resources
import os
import time
import datetime
import cgi
import shutil
import re

import py
from _pytest.runner import TestReport

from py.xml import html
from py.xml import raw

from html import escape





def find_urls(text):
    return re.findall('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', text)


class LogHTML(object):
    def __init__(self, logfile, prefix):
        logfile = os.path.expanduser(os.path.expandvars(logfile))
        self.logfile = os.path.normpath(os.path.abspath(logfile))
        self.project_root = os.getcwd()
        self.screenshot_path = os.path.join(os.path.dirname(self.logfile), 'screenshots')
        self.debug_event_path = os.path.join(os.path.dirname(self.logfile), 'debug_events')
        self.performance_path = os.path.join(os.path.dirname(self.logfile), 'performance_logs')
        self.used_screens = []
        self.used_debug_events = []
        self.prefix = prefix
        self.current_test_info = dict.fromkeys(['package', 'class', 'name'])
        self.current_test_reports = {}
        self.tests = []
        self.passed = self.skipped = 0
        self.failed = self.errors = 0
        self.test_logs = []
        self.errors = self.failed = 0
        self.passed = self.skipped = 0
        self.xfailed = self.xpassed = 0
        self.resources = ('style.css', 'jquery.js', 'main.js')

    def _make_report_dir(self):
        logfile_dirname = os.path.dirname(self.logfile)
        if logfile_dirname and not os.path.exists(logfile_dirname):
            os.makedirs(logfile_dirname)
        # copy across the static resources
        for file in self.resources:
            shutil.copyfile(
                pkg_resources.resource_filename(
                    __name__, os.path.sep.join(['resources', file])),
                os.path.abspath(os.path.join(logfile_dirname, file)))
        return logfile_dirname

    def process_testnames_from_report(self, test_node_path):
        names = [x.replace(".py", "") for x in test_node_path.split("::") if x != '()']
        self.current_test_info['package'] = names[0].replace("/", '.')
        self.current_test_info['class'] = names[1]
        self.current_test_info['name'] = names[2]
        return names

    def _write_captured_output(self, report):
        sec = dict(report.sections)
        output = ""
        for type in ('out', 'err'):
            for section in ('setup', 'call', 'teardown'):
                content = sec.get("Captured std{} {}".format(type, section))
                if content:
                    output += content
        return output

    def process_screenshot_files(self):
        if not os.path.exists(self.screenshot_path):
            os.makedirs(self.screenshot_path)

        for screenshot in self.used_screens:
            if os.path.exists(screenshot):
                shutil.copy(screenshot, self.screenshot_path)
        screenshot_folder = os.path.join(self.project_root, 'screenshots')
        if os.path.exists(screenshot_folder):
            shutil.rmtree(screenshot_folder)

    def process_debug_event_files(self):
        if not os.path.exists(self.debug_event_path):
            os.makedirs(self.debug_event_path)

        for event in self.used_debug_events:
            if os.path.exists(event):
                shutil.copy(event, self.debug_event_path)
        debug_event_folder = os.path.join(self.project_root, 'debug_events')
        if os.path.exists(debug_event_folder):
            shutil.rmtree(debug_event_folder)

    def process_performance_files(self):
        # TODO: The file processing code smells, need to refactor it to be more DRY :)
        if not os.path.exists(self.performance_path):
            os.makedirs(self.performance_path)

        performance_tmp_folder = os.path.join(self.project_root, 'performance_logs')
        perf_log_files = glob.glob(os.path.join(performance_tmp_folder, '*.log'))

        for perf_log_file in perf_log_files:
            shutil.copy(perf_log_file, self.performance_path)

        if os.path.exists(performance_tmp_folder):
            shutil.rmtree(performance_tmp_folder)

    def append_pass(self, report):
        self.passed += 1
        self._appendrow('Passed', report)

    def append_failure(self, report):
        if "xfail" in report.keywords:
            self._appendrow('XPassed', report)
            self.xpassed += 1
        else:
            self._appendrow('Failed', report)
            self.failed += 1

    def append_error(self, report):
        self._appendrow('Error', report)
        self.errors += 1

    def append_skipped(self, report):
        if "xfail" in report.keywords:
            self._appendrow('XFailed', report)
            self.xfailed += 1
        else:
            self._appendrow('Skipped', report)
            self.skipped += 1

    def pytest_runtest_logreport(self, report):
        self.process_testnames_from_report(report.nodeid)

        # It is the first report for that test
        current_test = self.current_test_info['name']
        if current_test not in self.current_test_reports:
            self.current_test_reports[current_test] = dict.fromkeys(['setup', 'call', 'teardown'])

        current_test_reports = self.current_test_reports[current_test]

        # Save the reports for all three test phases
        current_test_reports[report.when] = report  # type: TestReport

        if report.when == 'teardown': #finish processing the test
            setup_report = current_test_reports['setup']  # type: TestReport
            call_report = current_test_reports['call']  # type: TestReport
            teardown_report = current_test_reports['teardown']  # type: TestReport

            if setup_report.failed or teardown_report.failed:
                self.append_error(setup_report)

            elif setup_report.skipped:
                self.append_skipped(setup_report)

            elif call_report.skipped:
                self.append_skipped(call_report)

            elif call_report.passed:
                self.append_pass(call_report)

            elif call_report.failed:
                self.append_failure(call_report)


    def pytest_sessionstart(self):
        self.suite_start_time = time.time()

    def pytest_terminal_summary(self, terminalreporter):
        terminalreporter.write_sep("-", "generated html file: %s" % (self.logfile))

    def pytest_sessionfinish(self, session, exitstatus):
        self._make_report_dir()
        logfile = py.std.codecs.open(self.logfile, 'w', encoding='utf-8')

        suite_stop_time = time.time()
        suite_time_delta = suite_stop_time - self.suite_start_time
        numtests = self.passed + self.failed
        generated = datetime.datetime.now()
        doc = html.html(
            html.head(
                html.meta(charset='utf-8'),
                html.title('Test Report'),
                html.link(rel='stylesheet', href='style.css'),
                html.script(src='jquery.js'),
                html.script(src='main.js')),

            html.body(
                html.p('Report generated on %s at %s ' % (
                    generated.strftime('%d-%b-%Y'),
                    generated.strftime('%H:%M:%S'),
                )),
                html.div([html.p(
                    html.span('%i tests' % numtests, class_='all clickable'),
                    ' ran in %i seconds.' % suite_time_delta,
                    html.br(),
                    html.span('%i passed' % self.passed, class_='passed clickable'), ', ',
                    html.span('%i skipped' % self.skipped, class_='skipped clickable'), ', ',
                    html.span('%i failed' % self.failed, class_='failed clickable'), ', ',
                    html.span('%i errors' % self.errors, class_='error clickable'), '.',
                    html.br(), ),
                    html.span('Hide all errors', class_='clickable hide_all_errors'), ', ',
                    html.span('Show all errors', class_='clickable show_all_errors'),
                ], id='summary-wrapper'),
                html.div(id='summary-space'),
                html.table([
                    html.thead(html.tr([
                        html.th('Result', class_='sortable', col='result'),
                        html.th('Class', class_='sortable', col='class'),
                        html.th('Name', class_='sortable', col='name'),
                        html.th('Duration', class_='sortable numeric', col='duration'),
                        # html.th('Output')]), id='results-table-head'),
                        html.th('Links')]), id='results-table-head'),
                    html.tbody(*self.test_logs, id='results-table-body')], id='results-table')))
        logfile.write(
            '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">' + doc.unicode(
                indent=2))
        logfile.close()
        self.process_screenshot_files()
        self.process_debug_event_files()
        self.process_performance_files()

    def _appendrow(self, result, report):
        testclass = self.prefix + " " + self.current_test_info['package'] + self.current_test_info['class']
        testmethod = self.current_test_info['name']

        duration = getattr(report, 'duration', 0.0)

        additional_html = []
        links_html = []

        log = html.div(class_='log')

        if report.passed:
            log.append('Your tests are passing, but you are still curious. I like it :)')

        else:

            if hasattr(report, 'wasxfail'):
                self._append_xpath_section(log, report)

            if not report.skipped:

                self._append_stacktrace_section(log, report)

                links_html.append(self._link_to_debug_event(testmethod, log))

        self._append_screenshot(testmethod, log)

        output = self._append_captured_output(log, report)

        additional_html.append(log)

        links_html.append(self._link_to_browserstack_log(output))

        self.test_logs.append(html.tr([
            html.td(result, class_='col-result'),
            html.td(testclass, class_='col-class'),
            html.td(testmethod, class_='col-name'),
            html.td(round(duration), class_='col-duration'),
            html.td(links_html, class_='col-links'),
            html.td(additional_html, class_='debug')],
            class_=result.lower() + ' results-table-row'))

    def _link_to_browserstack_log(self, output):
        info = output.split(" ")
        link_html = []
        for i in range(0, len(info)):
            match_obj = re.search(r'(https?://www.browserstack.com/automate/builds/[\w]*/sessions/[\w]*)/', info[i])
            if match_obj:
                link_html.append(html.a("Browserstack", href=match_obj.group(1), target='_blank'))
                link_html.append(' ')
        return link_html

    def _link_to_debug_event(self, name, log):
        name = re.sub('[^A-Za-z0-9_.]+', '_', name)
        browser_name = self.prefix.split(",")[0].replace('[', '').lower()

        source = os.path.join(self.project_root, 'debug_events', browser_name + '_' + name + '.json')
        self.used_debug_events.append(source)
        return [html.a("Console log", href='debug_events/' + browser_name + '_' + name + '.json'), ' ']

    def _append_captured_output(self, log, report):
        # Use the output section from the "teardown" report - as it has all the previous sections (setup, call) as well
        test_name = self.current_test_info['name']
        output = self._write_captured_output(self.current_test_reports[test_name]['teardown'])
        log.append(html.h3('Captured output'))
        stacktrace_p = html.p(class_='stacktrace')
        stacktrace_p.append(output)
        log.append(stacktrace_p)
        return output

    @staticmethod
    def _append_stacktrace_section(log, report):
        log.append(html.h3('Stacktrace'))
        stacktrace_p = html.p(class_='stacktrace')
        for line in str(report.longrepr).splitlines():
            separator = line.startswith('_ ' * 10)
            if separator:
                stacktrace_p.append(line[:80])
            else:
                exception = line.startswith("E   ")
                if exception:
                    stacktrace_p.append(html.span(raw(escape(line)),
                                                  class_='error'))
                else:
                    stacktrace_p.append(raw(escape(line)))
            stacktrace_p.append(html.br())
        log.append(stacktrace_p)

    @staticmethod
    def _append_xpath_section(log, report):
        log.append(html.h3('Expected failure'))
        xfail_p = html.p(class_='xfail')
        xfail_reason = report.wasxfail
        xfail_p.append("Reason: ")
        # Does xfail reason contain e.g. link to JIRA?
        urls = find_urls(xfail_reason)
        if len(urls) > 0:
            xfail_p.append(html.a(xfail_reason, href=urls[0], target='_blank'))
        else:
            xfail_p.append(xfail_reason)
        log.append(xfail_p)

    def _append_screenshot(self, name, log):
        name = re.sub('[^A-Za-z0-9_.]+', '_', name)
        browser_name = self.prefix.split(",")[0].replace('[', '').lower()

        log.append(html.h3('Screenshots'))

        # Following works only for manually captured images.
        images_saved = os.path.join(self.project_root, 'screenshots')
        if os.path.isdir(images_saved):
            for file in os.listdir(images_saved):
                image_path = os.path.join(images_saved, file)
                if name in image_path:
                    self.used_screens.append(image_path)
                    # use relative path in img src
                    source = image_path.replace(self.project_root, '.')
                    log.append(source)
                    log.append(html.br())
                    log.append(html.img(src=source))
                    log.append(html.br())





from shishito.runtime.platform.shishito_execution import ShishitoExecution


class ControlExecution(ShishitoExecution):
    """ ControlExecution for mobile platform"""

    def get_test_result_prefix(self, config_section):
        """ Create string prefix for test results.

        :param str config_section: section in platform/environment.properties config
        :return: str with test result prefix
        """
        test_env = self.shishito_support.get_opt('test_environment')
        if(test_env == 'browserstack'):
            platform = self.shishito_support.get_opt(config_section, 'platform')
            device = self.shishito_support.get_opt(config_section, 'device')
            browser = self.shishito_support.get_opt(config_section, 'browser')

            return '[%s, %s, %s, %s]' % (config_section, platform, device, browser)
        if(test_env == 'appium'):
            platform = self.shishito_support.get_opt(config_section, 'platformName')
            platform_version = self.shishito_support.get_opt(config_section, 'platformVersion')

            return '[%s, %s, %s]' % (config_section, platform, platform_version)

        if (test_env == 'appium_bs'):
            os_version = self.shishito_support.get_opt(config_section, 'os_version')
            device = self.shishito_support.get_opt(config_section, 'device')
            return '[%s, %s, %s]' % (config_section, os_version, device)

